@@ -1,4 +1,4 @@
-## Copyright (C) 2019-2020 John Donoghue <john.donoghue@ieee.org>
+## Copyright (C) 2019-2021 John Donoghue <john.donoghue@ieee.org>
 ## 
 ## This program is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
@@ -14,120 +14,114 @@
 ## along with this program.  If not, see
 ## <https://www.gnu.org/licenses/>.
 
-## -*- texinfo -*- 
-## @deftypefn {} {@var{msg} =} midimsg (0)
-## @deftypefnx {} {@var{msg} =} midimsg (@var{type} ....)
-## @deftypefnx {} {@var{msg} =} midimsg ("note", @var{channel}, @var{note}, @var{velocity}, @var{duration}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("noteon", @var{channel}, @var{note}, @var{velocity}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("noteoff", @var{channel}, @var{note}, @var{velocity}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("programchange", @var{channel}, @var{prog}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("controlchange", @var{channel}, @var{ccnum}, @var{ccval}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("polykeypressure", @var{channel}, @var{note}, @var{keypressure}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("channelpressure", @var{channel}, @var{chanpressure}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("localcontrol", @var{channel}, @var{localcontrol}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("pitchbend", @var{channel}, @var{pitchchange}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("polyon", @var{channel}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("monoon", @var{channel}, @var{monochannels}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("omnion", @var{channel}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("omnioff", @var{channel}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("allsoundoff", @var{channel}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("allnotesoff", @var{channel}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("resetallcontrollers", @var{channel}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("start", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("stop", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("continue", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("systemreset", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("activesensing", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("timingclock", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("systemexclusive", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("systemexclusive", @var{bytes}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("eox", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("data", @var{bytes}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("songselect", @var{song}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("songpositionpointer", @var{songposition}, @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("tunerequest", @var{timestamp})
-## @deftypefnx {} {@var{msg} =} midimsg ("miditimecodequarterframe", @var{timeseq}, @var{timevalue}, @var{timestamp})
-## Create a midimsg object
-##
-## If the input parameter is 0, create an empty midi message object
-## Otherwise the first variable is the type of message to create, followed by the additional
-## parameters for the message.
-##
-## For each message type, the timestamp value is optional.
-##
-## @subsubheading Inputs
-## @var{type} - string message type or a midimsgtype.@*
-## @var{timestamp} - optional seconds time stamp for the event@*
-## @var{channel} - the channel to use for the message (1..16)@*
-## @var{note} - the value of the note to play/stop@*
-## @var{velocity} - the velocity value for a note on/off, with 0 stopping a note from sounding.@*
-## @var{duration} - seconds between starting and stopping a note when created a 'note' message.@*
-## @var{prog} - program number when doing a program change message.@*
-## @var{ccnum} - control change control number.@*
-## @var{ccval} - control change control value.@*
-## @var{keypressure} - key pressure value when creating a key pressure message.@*
-## @var{chanpressure} - channel pressure value when creating a channelpressure message.@*
-## @var{pitchchange} - pitch change value when creating a pitch bend message.@*
-## @var{localcontrol} - boolean value when creating a localcontrol message.@*
-## @var{monochannels} - channels specified for a mono on message.@*
-## @var{bytes} - array of data in range of 0 to 127 specified as part of a data message or
-## system exclusive message.@*
-## @var{song} - song selection number for a song selection message.@*
-## @var{songposition} - song position value for a song position message.@*
-## @var{timeseq} - timecode sequence number for a miditimecodequarterframe message.@*
-## @var{timevalue} - timecode value number for a miditimecodequarterframe message.@*
-##
-## @subsubheading Outputs
-## @var{msg} - a midimsg object containing the midi data of the message
-##
-## @subsubheading Properties
-## @var{timestamp} - timestamp of the message, or an array or timestamps if the the message is a
-## compound message.@*
-## @var{msgbytes} - the raw message bytes that make up the MIDI message.@*
-## @var{nummsgbytes} - the number of message bytes that make up the MIDI message.@*
-## @var{type} - string or midimsgtype that represents the message type.@*
-## @var{channel} - the channel number for message.@*
-## @var{note} - the note value for message (Only valid for noteon/off and polykeypressure).@*
-## @var{velocity} - the velocity value for message (Only valid for noteon/off).@*
-## @var{keypressure} - the keypressure value for message (Only valid for polykeypressure).@*
-## @var{channelpressure} - the chanpressure value for message (Only valid for channelpressure).@*
-## @var{localcontrol} - the localcontrol value for message (Only valid for localcontrol messages).@*
-## @var{monochannels} - channels specified for a mono on message.@*
-## @var{program} - program number specified for a program change message.@*
-## @var{ccnumber} - control change number specified for a control change message.@*
-## @var{ccvalue} - control change value specified for a control change message.@*
-## @var{song} - song number for a song selection message.@*
-## @var{songposition} - song position value for a song position message.@*
-## @var{pitchchange} - pitch change value for a pitch bend message.@*
-## @var{timecodesequence} - timecode sequence number for a miditimecodequarterframe message.@*
-## @var{timecodevalue} - timecode value number for a miditimecodequarterframe message.@*
-##
-## @subsubheading Examples
-## Create a note on/off pair with a duration of 1.5 seconds
-## @example
-## @code {
-## msg = midimsg('note', 1, 60, 100, 1.5)
-## }
-## @end example
-##
-## Create a separate note on/off pair with a time between them of 1.5 seconds
-## @example
-## @code {
-## msg = [midimsg('noteon', 1, 60, 100, 0), midimsg('noteoff', 1, 60, 0, 1.5)]
-## }
-## @end example
-##
-## Create a system reset message
-## @example
-## @code {
-## msg = midimsg('systemreset')
-## }
-## @end example
-##
-## @seealso{midifileread, midisend, midireceive, midimsgtype}
-## @end deftypefn
-
 classdef midimsg
+  ## -*- texinfo -*- 
+  ## @deftypefn {} {@var{msg} =} midimsg (0)
+  ## @deftypefnx {} {@var{msg} =} midimsg (@var{type} ....)
+  ## @deftypefnx {} {@var{msg} =} midimsg ("note", @var{channel}, @var{note}, @var{velocity}, @var{duration}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("noteon", @var{channel}, @var{note}, @var{velocity}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("noteoff", @var{channel}, @var{note}, @var{velocity}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("programchange", @var{channel}, @var{prog}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("controlchange", @var{channel}, @var{ccnum}, @var{ccval}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("polykeypressure", @var{channel}, @var{note}, @var{keypressure}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("channelpressure", @var{channel}, @var{chanpressure}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("localcontrol", @var{channel}, @var{localcontrol}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("pitchbend", @var{channel}, @var{pitchchange}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("polyon", @var{channel}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("monoon", @var{channel}, @var{monochannels}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("omnion", @var{channel}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("omnioff", @var{channel}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("allsoundoff", @var{channel}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("allnotesoff", @var{channel}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("resetallcontrollers", @var{channel}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("start", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("stop", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("continue", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("systemreset", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("activesensing", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("timingclock", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("systemexclusive", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("systemexclusive", @var{bytes}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("eox", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("data", @var{bytes}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("songselect", @var{song}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("songpositionpointer", @var{songposition}, @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("tunerequest", @var{timestamp})
+  ## @deftypefnx {} {@var{msg} =} midimsg ("miditimecodequarterframe", @var{timeseq}, @var{timevalue}, @var{timestamp})
+  ## Create a midimsg object
+  ##
+  ## If the input parameter is 0, create an empty midi message object
+  ## Otherwise the first variable is the type of message to create, followed by the additional
+  ## parameters for the message.
+  ##
+  ## For each message type, the timestamp value is optional.
+  ##
+  ## @subsubheading Inputs
+  ## @var{type} - string message type or a midimsgtype.@*
+  ## @var{timestamp} - optional seconds time stamp for the event@*
+  ## @var{channel} - the channel to use for the message (1..16)@*
+  ## @var{note} - the value of the note to play/stop@*
+  ## @var{velocity} - the velocity value for a note on/off, with 0 stopping a note from sounding.@*
+  ## @var{duration} - seconds between starting and stopping a note when created a 'note' message.@*
+  ## @var{prog} - program number when doing a program change message.@*
+  ## @var{ccnum} - control change control number.@*
+  ## @var{ccval} - control change control value.@*
+  ## @var{keypressure} - key pressure value when creating a key pressure message.@*
+  ## @var{chanpressure} - channel pressure value when creating a channelpressure message.@*
+  ## @var{pitchchange} - pitch change value when creating a pitch bend message.@*
+  ## @var{localcontrol} - boolean value when creating a localcontrol message.@*
+  ## @var{monochannels} - channels specified for a mono on message.@*
+  ## @var{bytes} - array of data in range of 0 to 127 specified as part of a data message or
+  ## system exclusive message.@*
+  ## @var{song} - song selection number for a song selection message.@*
+  ## @var{songposition} - song position value for a song position message.@*
+  ## @var{timeseq} - timecode sequence number for a miditimecodequarterframe message.@*
+  ## @var{timevalue} - timecode value number for a miditimecodequarterframe message.@*
+  ##
+  ## @subsubheading Outputs
+  ## @var{msg} - a midimsg object containing the midi data of the message
+  ##
+  ## @subsubheading Properties
+  ## @var{timestamp} - timestamp of the message, or an array or timestamps if the the message is a
+  ## compound message.@*
+  ## @var{msgbytes} - the raw message bytes that make up the MIDI message.@*
+  ## @var{nummsgbytes} - the number of message bytes that make up the MIDI message.@*
+  ## @var{type} - string or midimsgtype that represents the message type.@*
+  ## @var{channel} - the channel number for message.@*
+  ## @var{note} - the note value for message (Only valid for noteon/off and polykeypressure).@*
+  ## @var{velocity} - the velocity value for message (Only valid for noteon/off).@*
+  ## @var{keypressure} - the keypressure value for message (Only valid for polykeypressure).@*
+  ## @var{channelpressure} - the chanpressure value for message (Only valid for channelpressure).@*
+  ## @var{localcontrol} - the localcontrol value for message (Only valid for localcontrol messages).@*
+  ## @var{monochannels} - channels specified for a mono on message.@*
+  ## @var{program} - program number specified for a program change message.@*
+  ## @var{ccnumber} - control change number specified for a control change message.@*
+  ## @var{ccvalue} - control change value specified for a control change message.@*
+  ## @var{song} - song number for a song selection message.@*
+  ## @var{songposition} - song position value for a song position message.@*
+  ## @var{pitchchange} - pitch change value for a pitch bend message.@*
+  ## @var{timecodesequence} - timecode sequence number for a miditimecodequarterframe message.@*
+  ## @var{timecodevalue} - timecode value number for a miditimecodequarterframe message.@*
+  ##
+  ## @subsubheading Examples
+  ## Create a note on/off pair with a duration of 1.5 seconds
+  ## @example
+  ## msg = midimsg('note', 1, 60, 100, 1.5)
+  ## @end example
+  ##
+  ## Create a separate note on/off pair with a time between them of 1.5 seconds
+  ## @example
+  ## msg = [midimsg('noteon', 1, 60, 100, 0), midimsg('noteoff', 1, 60, 0, 1.5)]
+  ## @end example
+  ##
+  ## Create a system reset message
+  ## @example
+  ## msg = midimsg('systemreset')
+  ## @end example
+  ##
+  ## @seealso{midifileread, midisend, midireceive, midimsgtype}
+  ## @end deftypefn
+
   properties (SetAccess = private, GetAccess = public)
    data = {};
    timestamp = {};
@@ -197,7 +191,7 @@ classdef midimsg
           note = this.check_value127("note", varargin{2});
           vel = this.check_value127("velocity", varargin{3});
           dur = varargin{4};
-	  if !isscalar(dur) || !isnumeric(dur) || dur < 0
+          if !isscalar(dur) || !isnumeric(dur) || dur < 0
             error ('note expects at least duration to be a number >= 0')
           endif
           timestamp = 0;
@@ -291,8 +285,8 @@ classdef midimsg
           # pitch is 0 .. 16383 where 8120 is no change
           # pitch = uint16(varargin{2} + 0x2000);
           pitch = uint16(this.check_value16383("pitchchange", varargin{2}));
-          pitchlo = bitand(pitch, 0x7F);
-          pitchhi = bitand(bitshift(pitch, -7), 0x7f);
+          pitchlo = bitand(pitch, uint16(0x7F));
+          pitchhi = bitand(bitshift(pitch, -7), uint16(0x7f));
           timestamp = 0;
           if nargin > 3
             timestamp = this.check_timestamp(varargin{3});
@@ -410,8 +404,8 @@ classdef midimsg
           endif
           timestamp = 0;
           songpos = uint16(this.check_value16383("songposiition", varargin{1}));
-          songlo = bitand(songpos, 0x7F);
-          songhi = bitand(bitshift(songpos, -7), 0x7f);
+          songlo = bitand(songpos, uint16(0x7F));
+          songhi = bitand(bitshift(songpos, -7), uint16(0x7f));
           if nargin > 2
             timestamp = this.check_timestamp(varargin{2});
           endif
@@ -545,19 +539,42 @@ classdef midimsg
             this.timestamp{end+1} = timestamp;
           endif
 
+        case "metaevent"
+          if nargin < 3
+            error ('metaevent expects at least metatype and data')
+          endif
+          timestamp = 0;
+          if nargin > 3
+            timestamp = this.check_timestamp(varargin{3});
+          endif
+          # TODO: could be a integer or char type for metatype
+          event  = this.check_value127("metatype", varargin{1});
+          data  = uint8(varargin{2});  # TODO: check validity of the data <= 127
+          datasize = midimsg.makevariable(length(data));
+          this.data{end+1} = uint8([0xFF event datasize data]);
+          this.timestamp{end+1} = timestamp;
+
         otherwise
           error ("Unknown midi type '%s", typev);
       endswitch
        
     endfunction
 
-    function a = horzcat (a, b)
-      if !isa(a, 'midimsg') || !isa(b, 'midimsg')
+    function a = horzcat (a, varargin)
+      
+      if !isa(a, 'midimsg')
         error ("Cannot concatenate non midimsg elements");
       endif
-      for i=1:length (b.data)
-        a.data{end+1} = b.data{i};
-        a.timestamp{end+1} = b.timestamp{i};
+
+      for i = 1:nargin-1
+        b = varargin{i};
+        if !isa(b, 'midimsg')
+          error ("Cannot concatenate non midimsg elements");
+        endif
+        for idx=1:length (b.data)
+          a.data{end+1} = b.data{idx};
+          a.timestamp{end+1} = b.timestamp{idx};
+        endfor
       endfor
     endfunction
 
@@ -660,8 +677,8 @@ classdef midimsg
               endif
 
               songpos = uint16(this.check_value16383("songposition", rhs));
-              songlo = bitand(songpos, 0x7F);
-              songhi = bitand(bitshift(songpos, -7), 0x7f);
+              songlo = bitand(songpos, uint16(0x7F));
+              songhi = bitand(bitshift(songpos, -7), uint16(0x7f));
 
               data(2) = songlo;
               data(3) = songhi;
@@ -676,8 +693,8 @@ classdef midimsg
               endif
 
               pitchchange = uint16(this.check_value16383("pitchchange", rhs));
-              pitchlo = bitand(pitchchange, 0x7F);
-              pitchhi = bitand(bitshift(pitchchange, -7), 0x7f);
+              pitchlo = bitand(pitchchange, uint16(0x7F));
+              pitchhi = bitand(bitshift(pitchchange, -7), uint16(0x7f));
 
               data(2) = pitchlo;
               data(3) = pitchhi;
@@ -690,7 +707,7 @@ classdef midimsg
               if cmd != 0xF1
                 error ("timecodesequence property only valid for miditimecodequaterframe messages");
               endif
-	  
+  
               seq = bitshift(data(2), -3);
               val = bitand(data(2), 7);
 
@@ -706,7 +723,7 @@ classdef midimsg
               if cmd != 0xF1
                 error ("value property only valid for miditimecodequaterframe messages");
               endif
-	  
+  
               seq = bitshift(data(2), -3);
               val = bitand(data(2), 7);
 
@@ -719,14 +736,14 @@ classdef midimsg
 
             otherwise
               error("unimplemented midimsg.subsasgn property '%s'", s(1).subs);
-	  endswitch
+          endswitch
 
         case "()"
           idx = s(1).subs;
           if (numel (idx) != 1)
             error ("@midimsg/subsasgn: needs exactly one index");
           endif
-          if numel (s) == 1		
+          if numel (s) == 1
             # assign a value to here - so verify is a midimsg
             if !isa(rhs, "midimsg")
               error ("midimsg.subsasgn rhs of indexed value must be a midimsg");
@@ -737,7 +754,7 @@ classdef midimsg
             val = midimsg.createMessage(this.data{idx{1}}, this.timestamp{idx{1}});
             val = subsasgn (val, s(2:end), rhs);
           endif
-	  # store the modded data back in our object
+          # store the modded data back in our object
           this.data{idx{1}} = val.data{1};
           this.timestamp{idx{1}} = val.timestamp{1};
  
@@ -757,14 +774,19 @@ classdef midimsg
           if (numel (idx) != 1)
             error ("@midimsg/subsref: need exactly one index");
           endif
-          val = midimsg.createMessage(this.data{idx{1}}, this.timestamp{idx{1}});
+          off=[idx{1}];
+          val = midimsg(0);
+          for ix=off
+            a = midimsg.createMessage(this.data{ix}, this.timestamp{ix});
+            val = [val a];
+          endfor
         case "."
           switch tolower(s(1).subs)
           case "timestamp"
             if length(this.timestamp) == 1
               val = this.timestamp{1};
             else
-              val = this.timestamp;
+              val = cell2mat(this.timestamp);
             endif
           case "msgbytes"
             if length(this.data) == 1
@@ -777,8 +799,8 @@ classdef midimsg
               val = length(this.data{1});
 
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  val = [val length(this.data{i})];
+                for idx = 2:length(this.data)
+                  val = [val length(this.data{idx})];
                 endfor
               endif
             else
@@ -791,8 +813,8 @@ classdef midimsg
               if length(this.data) > 1
                 # we cant override cellstr yes, so just use strings for multiples
                 val = {char(val)}; 
-                for i = 2:length(this.data)
-                  val{end+1} = this.type_str(this.data{i});
+                for idx = 2:length(this.data)
+                  val{end+1} = this.type_str(this.data{idx});
                 endfor
               endif
             else
@@ -803,8 +825,8 @@ classdef midimsg
               data = this.data{1};
               val = bitand(data(1), 0x0F) + 1;
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   val = [val (bitand(data(1), 0x0F)+1)];
                 endfor
               endif
@@ -820,8 +842,8 @@ classdef midimsg
                 error ("note property only valid for noteon/off and polykeypressure");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0x80 || cmd == 0x90 || cmd == 0xA0
                     val = [val data(2)];
@@ -842,8 +864,8 @@ classdef midimsg
                 error ("velocity property only valid for noteon/off");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0x80 || cmd == 0x90
                     val = [val data(3)];
@@ -864,8 +886,8 @@ classdef midimsg
                 error ("keypressure property only valid for polykeypressure");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xA0
                     val = [val data(3)];
@@ -887,8 +909,8 @@ classdef midimsg
                 error ("channelpressure property only valid for channelpressure");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xD0
                     val = [val data(2)];
@@ -910,8 +932,8 @@ classdef midimsg
                 error ("localcontrol property only valid for localcontrol messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xB0 && data(2) == 122
                     val = [val data(3)];
@@ -932,8 +954,8 @@ classdef midimsg
                 error ("monochannels property only valid for monoon messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xB0 && data(2) == 126
                     val = [val data(3)];
@@ -954,8 +976,8 @@ classdef midimsg
                 error ("program property only valid for programchange messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xC0
                     val = [val data(2)];
@@ -976,8 +998,8 @@ classdef midimsg
                 error ("ccnumber property only valid for controlchange messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xB0 && data(2) <= 119
                     val = [val data(2)];
@@ -998,8 +1020,8 @@ classdef midimsg
                 error ("ccvalue property only valid for controlchange messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xB0 && data(2) <= 119
                     val = [val data(3)];
@@ -1021,8 +1043,8 @@ classdef midimsg
                 error ("song property only valid for songselect messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = data(1);
                   if cmd == 0xF3
                     val = [val data(2)];
@@ -1045,8 +1067,8 @@ classdef midimsg
                 error ("songpostion property only valid for songpositionpointer messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = data(1);
                   if cmd == 0xF2
                     v = bitshift(int16(data(3)), 7) + int16(data(2));
@@ -1070,8 +1092,8 @@ classdef midimsg
                 error ("pitchchange property only valid for pitchbend messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = bitand(data(1), 0xF0);
                   if cmd == 0xF2
                     v = bitshift(int16(data(3)), 7) + int16(data(2));
@@ -1095,8 +1117,8 @@ classdef midimsg
                 error ("timecodesequence property only valid for miditimecodequarterframe messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = data(1);
                   if cmd == 0xF1
                     v = bitshift(data(2), -3);
@@ -1120,8 +1142,8 @@ classdef midimsg
                 error ("timecodevalue property only valid for miditimecodequarterframe messages");
               endif
               if length(this.data) > 1
-                for i = 2:length(this.data)
-                  data = this.data{i};
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
                   cmd = data(1);
                   if cmd == 0xF1
                     v = bitand(data(2), 7);
@@ -1134,6 +1156,73 @@ classdef midimsg
               val = double(val);
             endif
 
+          case "metatype"
+
+            if length(this.data) > 0
+              data = this.data{1};
+              cmd = data(1);
+              if cmd == 0xFF && length(data) > 1
+                val = data(2);
+              else
+                error ("metatype property only valid for metaevent messages");
+              endif
+              if length(this.data) > 1
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
+                  cmd = data(1);
+                  if cmd == 0xFF &&  length(data) > 1
+                    val = data(2);
+                    val = [val v];
+                  else
+                    error ("metatype property only valid for metaevent messages");
+                  endif
+                endfor
+              endif
+              val = double(val);
+            endif
+
+          case "metadata"
+
+            if length(this.data) > 0
+              data = this.data{1};
+              cmd = data(1);
+              len = length(data);
+              if cmd == 0xFF && len > 1
+                if len < 128
+                  val = data(4:len);
+                elseif len < 128*128
+                  val = data(5:len);
+                elseif len < 128*128*128
+                  val = data(6:len);
+                else
+                  val = data(7:len);
+                endif
+              else
+                error ("metadata property only valid for metaevent messages");
+              endif
+              if length(this.data) > 1
+                for idx = 2:length(this.data)
+                  data = this.data{idx};
+                  cmd = data(1);
+                  len = length(data);
+                  if cmd == 0xFF && len > 1
+                    if len < 128
+                      v = data(4:len);
+                    elseif len < 128*128
+                      v = data(5:len);
+                    elseif len < 128*128*128
+                      v = data(6:len);
+                    else
+                      v = data(7:len);
+                    endif
+                    val = [val v];
+                  else
+                    error ("metadata property only valid for metaevent messages");
+                  endif
+                endfor
+              endif
+            endif
+
           otherwise
             error("unimplemented midimsg.subsref property '%s'", s(1).subs);
           endswitch
@@ -1155,9 +1244,9 @@ classdef midimsg
 
       s_idx = s_idx';
 
-      for i = 1:numel(s_idx)
-        msg.timestamp{end+1} = this.timestamp{s_idx(i)};
-        msg.data{end+1} = this.data{s_idx(i)};
+      for idx = 1:numel(s_idx)
+        msg.timestamp{end+1} = this.timestamp{s_idx(idx)};
+        msg.data{end+1} = this.data{s_idx(idx)};
       endfor
 
     endfunction
@@ -1168,8 +1257,8 @@ classdef midimsg
       else
         out = "";
       endif
-      for i=1:length(this.data)
-        data =  this.data{i};
+      for idx=1:length(this.data)
+        data =  this.data{idx};
         cmd = data(1);
         types = this.type_str(data); 
 
@@ -1217,12 +1306,16 @@ classdef midimsg
           val = bitand(data(2), 7);
           msgtext = [msgtext sprintf(" TimeCodeSequence: %d TimeCodeValue: %d", seq, val)];
         endif
+        if strcmp(types, "MetaEvent")
+          metatype = data(2);
+          msgtext = [msgtext sprintf(" MetaType: %d", metatype)];
+        endif
 
-        msgtext = [msgtext sprintf(" Timestamp: %f", this.timestamp{i})];
+        msgtext = [msgtext sprintf(" Timestamp: %f", this.timestamp{idx})];
         msgtext = [msgtext sprintf(" [")];
         msgtext = [msgtext sprintf(" 0x%02X", data)];
         msgtext = [msgtext sprintf(" ]")];
-	if nargout == 0
+        if nargout == 0
           disp(["   "  msgtext]);
         else
           out = [out msgtext "\n"];
@@ -1378,14 +1471,17 @@ classdef midimsg
            v = midimsgtype.ActiveSensing;
          elseif cmd == 0xFF
            v = midimsgtype.SystemReset;
+           if length(data) > 1
+             v = midimsgtype.MetaEvent;
+           endif
          endif
          # depend  on other bytes ?
        otherwise
          if length(data) < 1
            v = midimsgtype.Undefined;
-	 else
+         else
            v = midimsgtype.Data;
-	 endif
+         endif
       endswitch
       
     endfunction
@@ -1403,11 +1499,50 @@ classdef midimsg
       endif
       if nargin < 2
         ts = 0;
+      elif !isnumeric(ts)
+        error ("Expected Timestamp as a number")
       endif
       msg = midimsg(0);
       msg.data{end+1} = data;
       msg.timestamp{end+1} = ts;
     endfunction
+
+    function msg = fromStruct (msgstruct)
+      # expects 'RawBytes' and 'Timestamp' in a struct
+      if !isstruct (msgstruct) || !isfield (msgstruct, "RawBytes") || !isfield (msgstruct, "Timestamp")
+        error ("Expected struct with RawBytes and Timestamp field");
+      endif
+
+      msg = midimsg.createMessage(uint8(msgstruct.RawBytes), msgstruct.Timestamp);
+    endfunction
+  endmethods
+
+  methods (Static = true)
+    # octave 5 wont allow us to call private funcs from the class
+    # so a copy of private/makevariable is here
+    function data = makevariable (value)
+      t = uint64(value);
+      if t < 128
+        v = uint8(t);
+        data = [v];
+      else
+        tmp = dec2bin(t);
+        while mod(length(tmp), 7) != 0
+          tmp = [ "0" tmp ];
+        endwhile
+        data = [];
+        for i=1:7:length(tmp)
+          v = tmp(i:i+6);
+          v = uint8(bin2dec(v));
+          if i < length(tmp) -7
+            v = 128 + v;
+          endif
+
+          data = [data v];
+        endfor
+      endif
+    endfunction
+
   endmethods
 
 endclassdef
@@ -1442,10 +1577,7 @@ endclassdef
 %! assert(length(t) == 2);
 %! assert(strcmp(t{1}, "NoteOn"))
 %! assert(strcmp(t{2}, "NoteOn"))
-%! t = a.timestamp;
-%! assert(length(t) == 2);
-%! assert(t{1}, 0.0)
-%! assert(t{2}, 1.2)
+%! assert(a.timestamp, [0.0 1.2]);
 %! assert(a.channel, [2 2]);
 
 %!test
@@ -1807,6 +1939,15 @@ endclassdef
 %! assert(c(2).note, 60);
 %! assert(c(2).velocity, 10);
 
+%!test
+%! a = midimsg("metaevent", 1, "hello");
+%! assert(isa(a, "midimsg"));
+%! assert(length(a) == 1);
+%! assert(a.type == "MetaEvent");
+%! assert(a.metatype, 1);
+%! assert(a.metadata, uint8([0x68 0x65 0x6C 0x6C 0x6F]));
+%! assert(a.msgbytes, uint8([0xFF 0x01 0x05 0x68 0x65 0x6C 0x6C 0x6F]))
+
 %!test
 %! # basic assign operations
 %!
@@ -1866,3 +2007,26 @@ endclassdef
 %! assert(a(2).channel, 12);
 %! assert(a(2).note, 60);
 
+%!test
+%! as = struct("RawBytes", [0x90 0x3C 0x14], "Timestamp", 10);
+%! a = midimsg.fromStruct(as);
+%! assert(length(a) == 1);
+%! assert(a.type == "NoteOn");
+%! assert(a.channel, 1);
+%! assert(a.timestamp, 10);
+%! assert(a.msgbytes, uint8([0x90 0x3C 0x14]));
+
+%!test
+%! a1 = midimsg("noteon", 1, 80, 100, 5);
+%! a2 = midimsg("noteon", 1, 81, 100, 1);
+%! a3 = midimsg("noteon", 1, 82, 100, 10);
+%! msgs = [a1 a2 a3];
+%! ts = msgs.Timestamp;
+%! [~, idx] = sort(ts);
+%! smsgs = msgs(idx);
+%! assert(smsgs(1).Timestamp, 1)
+%! assert(smsgs(2).Timestamp, 5)
+%! assert(smsgs(3).Timestamp, 10)
+%! assert(smsgs(1).Note, 81)
+%! assert(smsgs(2).Note, 80)
+%! assert(smsgs(3).Note, 82)
